// File ctype.h for CC1B
#ifndef _CTYPE_H_
#define _CTYPE_H_
//include other libraries here
#pragma library 1

/**
 * \file ctype.h
 * \brief Library for character functions
 *
 * This library provides character functions.
 * \n
 */

/**
 * \brief Test if character is alphabetic
 *
 * Test if character is alphabetic (a-z or A-Z).
 *
 * \param ch Character to be tested.
 * \return True if ch is alphabetic.
 * \n
 */
bit isalpha(char ch)
{
  return (ch<='z' && ch>='a') || (ch<='Z' && ch>='A');
}

/**
 * \brief Test if character is an ascii character
 *
 * Test if character is an ascii character (0-127).
 *
 * \param ch Character to be tested.
 * \return True if ch is an ASCII character (0-127).
 * \n
*/
bit isascii(char ch)
{
  return (ch < 128);
}

/**
 * \brief Test if character is a control character
 *
 * Test if character is a control character (0-31 or 127).
 *
 * \param ch Character to be tested.
 * \return True if ch is a control character.
 * \n
 */
bit iscntrl(char ch)
{
  return (ch <= 31) || (ch == 127);
}

/**
 * \brief Test if character is a digit
 *
 * Test if character is a digit (0-9).
 *
 * \param ch Character to be tested.
 * \return True if ch is a digit.
 * \n
 */
bit isdigit(char ch)
{
  return ch<='9' && ch>='0';
}

/**
 * \brief Test if character is alphanumeric
 *
 * Test if character is alphanumeric (a-z, A-Z or 0-9).
 *
 * \param ch Character to be tested.
 * \return True if ch is alphanumeric.
 * \n
 */
bit isalnum(char ch)
{
  bit result = isalpha(ch);
  result |= isdigit(ch);
  return result;
}

/**
 * \brief Test if character is a printable character other than a space
 *
 * Test if character is a printable character other than a space (33-126).
 *
 * \param ch Character to be tested.
 * \return True if ch is a printable character other than a space.
 * \n
 */
bit isgraph(char ch)
{
  return ch>=33 && ch<=126;
}

/**
 * \brief Test if character is a lowercase alphabetic
 *
 * Test if character is a lowercase alphabetic (a-z).
 *
 * \param ch Character to be tested.
 * \return True if ch is a lowercase alphabetic.
 * \n
 */
bit islower(char ch)
{
  return ch<='z' && ch>='a';
}

/**
 * \brief Test if character is a printable character (32-126)
 *
 * Test if character is a printable character (32-126).
 *
 * \param ch Character to be tested.
 * \return True if ch is a printable character.
 * \n
 */
bit isprint(char ch)
{
  return ch>=32 && ch<=126;
}

/**
 * \brief Test if character is a punctuation character
 *
 * Test if character is a punctuation character (all but control and alphanumeric).
 *
 * \param ch Character to be tested.
 * \return True if ch is a punctuation character.
 * \n
 */
bit ispunct(char ch)
{
  return !isalnum(ch) && !iscntrl(ch);
}

/**
 * \brief Test if character is a space, tab or newline
 *
 * Test if character is a space, tab or newline.
 *
 * \param ch Character to be tested.
 * \return True if ch is a space, tab or newline.
 * \n
 */
bit isspace(char ch)
{
  return ch==' ' || ch=='\t' || ch=='\n';
}

/**
 * \brief Test if character is an uppercase alphabetic
 *
 * Test if character is an uppercase alphabetic (A-Z).
 *
 * \param ch Character to be tested.
 * \return True if ch is an uppercase alphabetic.
 * \n
 */
bit isupper(char ch)
{
  return ch<='Z' && ch>='A';
}

/**
 * \brief Test if character is a hexadecimal digit
 *
 * Test if character is a hexadecimal digit (0-9, A-F, a-f).
 *
 * \param ch Character to be tested.
 * \return True if ch is a hexadecimal digit.
 * \n
 */
bit isxdigit(char ch)
{
  ch.5 = 0;
  return (ch<='9' && ch>='0') || (ch<='F' && ch>='A');
}

/**
 * \brief Convert character to equivalent ascii value
 *
 * Convert character to equivalent ascii value.
 *
 * \param ch Character to be converted.
 * \return ascii equivalent of ch.
 * \n
 */
char toascii(char ch)
{
  return (ch);
}

/**
 * \brief Convert character to lowercase
 *
 * Convert character to lowercase.
 *
 * \param ch Character to be converted.
 * \return lowercase of ch if uppercase, else ch.
 * \n
 */
char tolower(char ch)
{
  if (ch<='Z' && ch>='A') return (ch+32);
  return (ch);
}

/**
 * \brief Convert character to uppercase
 *
 * Convert character to uppercase.
 *
 * \param ch Character to be converted.
 * \return uppercase of ch if it is lowercase, else ch.
 * \n
 */
char toupper(char ch)
{
  if (ch<='z' && ch>='a') return (ch-32);
  return (ch);
}

#pragma library 0
#endif
// End of file ctype.h
